// Copyright (C) 2023 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

function populateData(listData) {
    listData.append({
                        name: "John Doe",
                        gender: "m",
                        date: "02/15/2017",
                        time: "09:20 am",
                        text: "I found the bug that has been bothering us for years! It was in your code segment :P"
                      });

    listData.append({
                        name: "Jane Worldege",
                        gender: "f",
                        date: "02/06/2017",
                        time: "10:15 am",
                        text: "John told me about your bug. One more such thing and you're fired!"
                      });

    listData.append({
                        name: "Jennifer Wang",
                        gender: "f",
                        date: "02/03/2017",
                        time: "05:16 pm",
                        text: "Hey! Have you seen the new Qt version? It can do 3D stuff now!"
                      });
}
