// Copyright (C) 2025 Jarek Kobus
// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

#ifndef RECIPE_H
#define RECIPE_H

#include <QTaskTree>

class GlueInterface;

QtTaskTree::ExecutableItem recipe(GlueInterface &iface);

#endif // RECIPE_H
