// Copyright (C) 2023 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

function getWalkingSteps() {
    return 4000.;
}

function getWalkingGoal() {
    return 8000.;
}

function getWalkingTime() {
    return 40.;
}

function getWalkingCalories() {
    return 100.;
}

function getRunningGoal() {
    return 8.;
}

function getRunningDistance() {
    return 4.;
}

function getRunningTime() {
    return 16.;
}

function getRunningCalories() {
    return 200.;
}
